#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/interrupt.h>
#include "nambenz.h"

static int16_t distance;
static volatile int count;
static int8_t state = RUNNING;

char get_data()
{	
	while (!(UCSR0A & (1<<RXC0)));
		return UDR0;
}

ISR(INT0_vect)
{
    count = TCNT1;
    TCNT1 = 0;
}

void init_usart()
{
	UCSR0A = 0;
	UCSR0B = _BV(RXEN0) | _BV(TXEN0); //RX, TX enable
	UCSR0C = _BV(UCSZ01) | _BV(UCSZ00); //8 data bit
 
	int baud = 103;
	UBRR0 = baud;
}

void sensor_init()
{
	/* Se initializeaza pinii de Trigger si Echo */
    DDRD |= (1 << PD4);
    DDRD &= ~(1 << PD2);
    EIMSK |= (1 << INT0);
    EICRA |= (1 << ISC00);
    TCCR1A = 0;
}

void check_sensor()
{
    PORTD |= (1 << PD4);
    _delay_us(15);
    PORTD &= ~(1 << PD4);
}

void init_led()
{
	DDRD |= (1 << PD7);
	PORTC &= ~(1 << PC0);
}

void stop_motor()
{
	PORTB &= ~(1 << PC4);
	PORTB &= ~(1 << PC5);
	PORTB &= ~(1 << PC6);
	PORTB &= ~(1 << PC7);
}

void check_bluetooth()
{
	char command;
	int isColliding = 0;
	int autopilot = false;
	int can_reverse_direction = true;
	int8_t rand = 0;

	DDRB |= (1 << PB4);
	DDRB |= (1 << PB5);

	DDRB |= (1 << PB6);
	DDRB |= (1 << PB7);

	PORTB &= ~(1 << PC4);
	PORTB &= ~(1 << PC5);
	PORTB &= ~(1 << PC6);
	PORTB &= ~(1 << PC7);

	while (1) {

		command = get_data();

		if (command == 'U')
		{
			/* Trecerea in pilot automat */
			stop_motor();
			_delay_ms(_stop_delay);
			autopilot = true;
		}

		check_sensor();

		distance = count / 15;
		isColliding = (distance <= MAX_DISTANCE);

		if (autopilot)
		{
			rand = (rand + 1) % 2;

			if (command == 'R')
			{
				/* Oprirea pilotului automat */
				stop_motor();
				_delay_ms(_stop_delay);
				autopilot = false;

				continue;
			}

			if (isColliding && state == RUNNING)
			{
				/* Masinuta intra in starea de REVERSE */
				PORTD |= (1 << PD7); /* Start led */
				stop_motor();

				state = REVERSE;
				_delay_ms(100);
				continue;
			}
			else if (isColliding && state == REVERSE)
			{
				// Go BACK
				PORTB |= (1 << PB6);

				/* decide which way to reverse */
				if (can_reverse_direction)
				{
					if (rand == reverse_left)
						PORTB |= (1 << PB5);
					else
						PORTB |= (1 << PB4);

					can_reverse_direction = false;
				}
			}
			else if (!isColliding && state == REVERSE)
			{
				/* Daca masina nu mai detecteaza obstacol */
				state = RUNNING;
				can_reverse_direction = true;

				PORTD &= ~(1 << PD7); /* Stop led */
				stop_motor();
			}
			else
			{
				PORTB |= (1 << PB7);
			}

			if (state == REVERSE)
				_delay_ms(95);
				
			_delay_ms(5);

		} else {
		
			if (command == 'L')
			{
				PORTB &= ~(1 << PB4);
				PORTB |= (1 << PB5);

				/* Deactivate directions */
				PORTB &= ~(1 << PB6);
				PORTB &= ~(1 << PB7);
				_delay_ms(_delay);

			}
			else if (command == 'R')
			{
				PORTB |= (1 << PB4);
				PORTB &= ~(1 << PB5);

				/* Deactivate directions */
				PORTB &= ~(1 << PB6);
				PORTB &= ~(1 << PB7);
				_delay_ms(_delay);
			}
			else if (command == 'B')
			{
				PORTB |= (1 << PB6);
				PORTB &= ~(1 << PB7);

				/* Deactivate directions */
				PORTB &= ~(1 << PB4);
				PORTB &= ~(1 << PB5);
				_delay_ms(_delay);
			}
			else if (command == 'F')
			{
				PORTB &= ~(1 << PB6);
				PORTB |= (1 << PB7);

				/* Deactivate directions */
				PORTB &= ~(1 << PB4);
				PORTB &= ~(1 << PB5);
				_delay_ms(_delay);
			}
			else if (command == 'G')
			{
				/* Forward Left */
				PORTB &= ~(1 << PB4);
				PORTB |= (1 << PB5);

				PORTB &= ~(1 << PB6);
				PORTB |= (1 << PB7);
				_delay_ms(_delay);

			}
			else if (command == 'I')
			{
				/* Forward Right */
				PORTB |= (1 << PB4);
				PORTB &= ~(1 << PB5);

				PORTB &= ~(1 << PB6);
				PORTB |= (1 << PB7);

				_delay_ms(_delay);
			}
			else if (command == 'H')
			{
				/* Back Left */
				PORTB &= ~(1 << PB4);
				PORTB |= (1 << PB5);

				PORTB |= (1 << PB6);
				PORTB &= ~(1 << PB7);
				_delay_ms(_delay);
			}
			else if (command == 'J')
			{
				/* Back Right */
				PORTB |= (1 << PB4);
				PORTB &= ~(1 << PB5);

				PORTB |= (1 << PB6);
				PORTB &= ~(1 << PB7);
				_delay_ms(_delay);
			}
			 else {
				stop_motor();
			}
		}
	}
}

int main(void) {

	sei();

	TCCR1B |= ((1 << CS10) | (1 << CS11));

	init_usart();
	sensor_init();
	init_led();
	check_bluetooth();
	
	return 0;
}
